import sys


class DictObj():
    def __init__(self, entries=None):
        if isinstance(entries, dict):
            for key, value in entries.items():
                if isinstance(entries[key], list):
                    value = [DictObj(item) if isinstance(item, dict) else item for item in value]
                if isinstance(entries[key], dict):
                    value = DictObj(value)
                setattr(self, ''.join(filter(str.isalnum, key)), value)

    def __delitem__(self, key):
        return delattr(self, key)

    def __getitem__(self, key):
        return getattr(self, key)

    def __iter__(self):
        return iter(self.__dict__)

    def __len__(self):
        return len(self.__dict__)

    def __repr__(self):
        return '<DictObj object>(' + repr(self.__dict__) + ')'

    def __setitem__(self, key, value):
        return setattr(self, key, value)
    
    def __str__(self):
        return str(self.__dict__)

    if sys.version_info >= (3, 8):
        def __reversed__(self):
            return reversed(self.__dict__)

    if sys.version_info >= (3, 9):
        def __class_getitem__(self, key):
            return self.__dict__.__class_getitem__(key)

        def __ior__(self, value):
            return self.__dict__.__ior__(value)

        def __or__(self, value):
            return self.__dict__.__or__(value)

    def clear(self):
        return self.__dict__.clear()

    def copy(self):
        return DictObj(self.__dict__.copy())

    def fromkeys(self, keys, value=None):
        return DictObj(self.__dict__.fromkeys(keys, value))

    def get(self, key, value=None):
        return self.__dict__.get(key, value)

    def items(self):
        return self.__dict__.items()

    def keys(self):
        return self.__dict__.keys()

    def pop(self, key, value=None):
        return self.__dict__.pop(key, value)
    
    def popitem(self):
        return self.__dict__.popitem()
    
    def setdefault(self, key, value=None):
        return self.__dict__.setdefault(key, value)

    def update(self, entries):
        return self.__init__(entries)

    def values(self):
        return self.__dict__.values()


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyduinocoin",
    version="1.0.5",
    author="Sergio Contreras Agustí (backrndsource)",
    author_email="backrndsource@gmail.com",
    description="pyDuinoCoin is a simple python integration for the DuinoCoin REST API, that allows developers to communicate with DuinoCoin Master Server.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/backrndsource/pyduinocoin",
    project_urls={
        "Bug Tracker": "https://github.com/backrndsource/pyduinocoin/issues",
    },
    keywords=["DUCO", "DuinoCoin", "api-rest", "python3", "duino", "duino-coin", "api", "client", "REST", "crypto", "coin"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    python_requires=">=3.5",
)



import requests
from .exceptions import PyDuinoCoinException
from .dict_obj import DictObj


class DuinoClient():
    '''
    API REST documentation:
    https://github.com/revoxhere/duco-rest-api
    '''

    def __init__(self):
        self._base_url = 'https://server.duinocoin.com/'

    def _get(self, endpoint, params='', results_key='result'):
        '''
        '''
        url = '{}{}'.format(self._base_url, endpoint)
        response = requests.request("GET", url, params=params)

        if response.status_code == 200:
            json = response.json()

            if ('success' in json and json['success'] == False) or 'message' in json:
                raise PyDuinoCoinException(json['message'] if 'message' in json else 'Unknown error')

            if results_key:
                if isinstance(json[results_key], list):
                    return [DictObj(item) if isinstance(item, dict) else item for item in json[results_key]]
                else:
                    return DictObj(json[results_key])

            return DictObj(json)
        else:
            raise PyDuinoCoinException('SERVER REQUEST ERROR (' + str(response.status_code) + ')')


    def user(self, username, limit=5):
        '''
        Return username's balance, last transactions and miners in one request.
        '''

        return self._get(f'users/{str(username)}', {'limit':limit})

    def auth(self, username, password):
        '''
        Check username's password.
        '''
        
        return self._get(f'auth/{str(username)}', {'password':password})

    def transactions(self, hash=None, limit=None):
        '''
        Return all transactions or an unique transaction with that hash.
        '''
        
        return self._get(f'transactions/{str(hash)}') if hash else self._get('transactions', {'limit':limit} if limit else {}) 

    def user_transactions(self, username, limit=10):
        '''
        Return transactions related to username.
        '''

        return self._get(f'user_transactions/{str(username)}', {'limit':limit})

    def id_transactions(self, id):
        '''
        Return a transaction with that id.
        '''

        return self._get(f'id_transactions/{str(id)}')

    def transfer(self, username, password, recipient, amount, memo='Powered by pyduinocoin'):
        '''
        Transfer funds from username to recipient.
        '''

        return self._get('transaction', {'username':username, 'password':password, 'recipient':recipient, 'amount':amount, 'memo':memo}, results_key=None)

    def miners(self, username=None, limit=None):
        '''
        Return all miners or username's miners.
        '''

        return self._get(f'miners/{str(username)}', {'limit':limit} if limit else {}) if username else self._get('miners', {'limit':limit} if limit else {})
    
    def balances(self, username=None, limit=None):
        '''
        Return all balances or username's balance.
        '''

        return self._get(f'balances/{str(username)}') if username else self._get('balances', {'limit':limit} if limit else {})

    def statistics(self):
        '''
        Return server statistics.
        '''

        return self._get('statistics', results_key=None)

    def all_pools(self):
        '''
        Return all non-hidden mining pools.
        '''

        return self._get('all_pools')

    def exchange_request(self, username, password, email, type, amount, coin, address):
        '''
        Submit exchange request in the DUCO Exchange.
        '''

        return self._get('exchange_request', {'username':username, 'password':password, 'email':email, 'type':type, 'amount':amount, 'coin':coin, 'address':address}, results_key=None)